#!/usr/bin/env python
__author__ = "Philippe May <phil.dev@philome.mooo.com>"
__license__ = "BSD"
__doc__ = "Listen to messages from the SMS Forwarder app on Android, and send mail."

import os
import textwrap
from datetime import datetime
from socket import gethostname
from email.message import EmailMessage

from fastapi import FastAPI
from aiosmtplib import send
from pydantic import BaseModel, Field, computed_field
from pydantic_settings import BaseSettings, SettingsConfigDict

USER = os.environ.get("USER", "root")
HOST_NAME = gethostname()


## Definition of settings:
class MailSettings(BaseSettings):
    model_config = SettingsConfigDict(env_prefix="SMS_HANDLER_MAIL_")
    enable: bool = True
    sender: str = f"{USER}@{HOST_NAME}"
    to: str = f"{USER}@{HOST_NAME}"
    subject: str = """SMS from {pretty_from}"""
    template: str = textwrap.dedent(
        """\
        {text}
        ---
        From: {pretty_from}
        Sent: {sentStamp:%c}
        Received: {receivedStamp:%c}
        Sim: {sim}
        """
    )


class MailServerSettings(BaseSettings):
    model_config = SettingsConfigDict(env_prefix="SMS_HANDLER_MAIL_SERVER_")
    hostname: str = "localhost"
    port: int = 25
    username: str | None = None
    password: str | None = None
    start_tls: bool = False
    use_tls: bool = False


class Settings(BaseSettings):
    model_config = SettingsConfigDict(env_prefix="SMS_HANDLER_")
    mail: MailSettings = MailSettings()
    mail_server: MailServerSettings = MailServerSettings()


class SmsAlert(BaseModel):
    from_: str = Field(alias="from")
    fromName: str | None = None
    text: str
    sentStamp: datetime
    receivedStamp: datetime
    sim: str

    @computed_field
    @property
    def pretty_from(self) -> str:
        if self.fromName is not None:
            return f"{self.fromName} ({self.from_})"
        else:
            return self.from_


settings = Settings()
app = FastAPI()


def send_mail(smsAlert: SmsAlert):
    msg = EmailMessage()
    fmt = smsAlert.model_dump()
    fmt["from"] = fmt.pop("from_")
    msg["Subject"] = settings.mail.subject.format_map(fmt)
    msg["From"] = settings.mail.sender
    msg["To"] = settings.mail.to
    msg.set_content(settings.mail.template.format_map(fmt))
    return send(
        msg,
        hostname=settings.mail_server.hostname,
        username=settings.mail_server.username,
        password=settings.mail_server.password,
        port=settings.mail_server.port,
        start_tls=settings.mail_server.start_tls,
        use_tls=settings.mail_server.use_tls,
    )


@app.post("/handle-sms")
async def handle_sms(smsAlert: SmsAlert):
    if settings.mail.enable:
        await send_mail(smsAlert)
    return "OK"


def main():
    from uvicorn import run
    from argparse import ArgumentParser

    parser = ArgumentParser(description=__doc__)
    parser.add_argument(
        "-l",
        "--host",
        type=str,
        default="0.0.0.0",
        help="Address to listen to (default: 0.0.0.0)",
    )
    parser.add_argument(
        "-p", "--port", type=int, default=8025, help="Port to listen to (default: 8025)"
    )
    parser.add_argument(
        "-v", "--version", action="store_true", help="Print version and exit"
    )
    args = parser.parse_args()

    if args.version:
        import sys
        from importlib.metadata import version

        print(version("sms_handler"))
        sys.exit(0)

    run(app, host=args.host, port=args.port)
